clear state
clear tmpChromosome
clear poplin
clear J_score
% GA parameters for training
my = 7;               % Population size
g_max = 3;             % Number of generation
p_c = 0.7;             % Crossover probability
p_m = 0.2;             % Mutation probability
state.populationSize = my;
state.crossoverProbability = p_c;
state.mutationProbability = p_m;
state.u_bound = 0.5;
state.l_bound = 0.05;
state.startTime = clock;

% Initialise population and score
poplin(1,1) = simIn.w_surge;
poplin(1,2) = simIn.w_sway;
poplin(1,3) = simIn.w_yaw;
poplin(2,1) = state.l_bound/2;
poplin(2,2) = state.l_bound;
poplin(2,3) = state.l_bound;
for k = 3:my
    poplin(k,1) = poplin(k-1,1) + (state.u_bound - state.l_bound)/my;
    poplin(k,2) = poplin(k,1);
    poplin(k,3) = poplin(k,1);
end
%state.Population = poplin;
state.Population = initPopNonLin(my, state.l_bound, state.u_bound);
state.Score = inf(my,3);

% Finding score for default values
sim('supply_',[0 simIn.time]);
%simIn.score = JT_lev(end)
J_score(1,:)=...
        [Jp_Jh_lev(end,1)*simIn.lambda(1) + JT_tau(end,1)*(1-simIn.lambda(1)),...
         Jp_Jh_lev(end,2)*simIn.lambda(2) + JT_tau(end,2)*(1-simIn.lambda(2)),...
         Jp_Jh_lev(end,3)*simIn.lambda(3) + JT_tau(end,3)*(1-simIn.lambda(3))];
% Setting population 1 to default values to compare with
state.Population(1,:) = [simIn.w_surge, simIn.w_sway, simIn.w_yaw];
state.Score(1,:) = J_score(1,:);
state.BestScore(3,g_max) = Inf;
state.crossovers(3,g_max) = 0;
state.mutations(3,g_max) = 0;
state.LastImprovement(3,1) = 1;

%% Running training for g_max generations
for dof = 1:3
    % For each dof?
    g = 0; % Initialise generation to 0
    stop = 0;
    state.Generation = g;
    % Running GA training
    while g < g_max && stop == 0 % legge inn fitness krav ogs
        % Update generation
        state.Generation = state.Generation + 1;
        % Fitness calculation
        for pop = 1:my
            population = state.Population(pop,:);
            % Defines controller parameter
            if dof == 1
                simIn.w_surge = state.Population(pop,1);
            elseif dof == 2
                simIn.w_surge = state.BestChromosome(1,end);
                simIn.w_sway = state.Population(pop,2);
            elseif dof == 3
                simIn.w_surge = state.BestChromosome(1,end);
                simIn.w_sway = state.BestChromosome(2,end);
                simIn.w_yaw = state.Population(pop,3);
            end
            % Run simulation to get Fitness value
            sim('supply_',[0 simIn.time]);
            J_score(pop,dof)=...
                    [Jp_Jh_lev(end,dof)*simIn.lambda(dof) + JT_tau(end,dof)*(1-simIn.lambda(dof))];%...
            state.Score(pop,:) = J_score(pop,:);
        end
        % remember the best score
        [best,i] = max(state.Score(:,dof));

        bestChromosome = state.Population(i,dof);
        generation = state.Generation;
        state.BestScore(dof,generation) = best;
        state.BestChromosome(dof,generation) = bestChromosome;

        % Tournament Selection of best population for this generation
        for m = 1:my
            l = ceil(rand(1)*my);
            o = ceil(rand(1)*my);
            if state.Score(l,dof)>= state.Score(o,dof)
                tmpChromosome(m,dof) = state.Population(l,dof);
            else
                tmpChromosome(m,dof) = state.Population(o,dof);
            end
        end

        % Crossover
        lambda = rand(1);   % Crossover parameter
        ccr = 0;        % counting crossover made
        for n = 1:my %ceil(populationSize*state.crossoverProbability/2)
            cross = rand(1);
            if cross < state.crossoverProbability
                l = ceil(rand(1)*my); % c1
                o = ceil(rand(1)*my); % c2
                tmpChromosome(l,dof) = lambda*tmpChromosome(l,dof) + (1-lambda)*tmpChromosome(o,dof);
                tmpChromosome(o,dof) = lambda*tmpChromosome(o,dof) + (1-lambda)*tmpChromosome(l,dof);
                ccr = ccr + 1;
            end
        end
        state.crossovers(dof,generation) = ccr;
        % Mutation
        mcr = 0;        % counting mutation made
        for n = 1:my %ceil(populationSize*state.crossoverProbability/2)
            mut = rand(1);
            if mut < state.mutationProbability
                l = ceil(rand(1)*(my-1));%c(l,:) chromomson
                o = ceil(rand(1)*3); %c(l,o) variable
                tmp = initPopNonLin(1, state.l_bound, state.u_bound);
                tmpChromosome(l,dof) = tmp(dof);%alternative is tmp(o)
                mcr = mcr +1;
            end
        end
        state.mutations(dof,generation) = mcr;
        % keep track of improvement in the best
        if((generation > 1)) && (state.BestScore(dof,generation-1) < best)
            state.LastImprovement(dof,1) = generation;
        end
        % keeping best chromosome as the first one
        tmpChromosome(1,dof) = bestChromosome;
        state.Population(:,dof) = tmpChromosome(:,dof);
        g = state.Generation;
        % stop = input('1 if stop, 0 if continue')
    end % inner while
end

state.stopTime = clock;
state.runTime = round(etime(state.stopTime, state.startTime)/10);
%% Set result
simIn.w_surge = state.BestChromosome(1,end);
simIn.w_sway = state.BestChromosome(2,end);
simIn.w_yaw = state.BestChromosome(3,end);