function levels_plot_pos(ns_e,ew_e,offset_su,offset_sw,psi_d,x_pos,y_pos,...
    L1Str,L2Str,L3Str,JpStr,JtsuStr,JtswStr,seaState,weight,w_surge,w_sway)
% Plot a figure of deviation levels and the sensor error ellipse
% deviation level = max sensor error + offset_su/sw
% ns_e = Sensor Error north-south in [m]
% ew_e = Sensor Error east-west in [m]
% offset_su = offset deviation in surge 3 levels,
% offset_sw = offset deviation in surge 3 levels;
% psi_d = setpoint heading in rad psi_d [0, 2pi)
% x_pos = vessels position in surge
% y_pos = vessels position in sway

% max sensor error
max_se = max(ns_e,ew_e);

% Setpoint heading
Str = ['\psi_d = ',num2str(psi_d*180/pi),'^o'];
if psi_d > pi
    psi = psi_d - pi;
else
    psi = psi_d;
end
 
su_e = max_se + offset_su(1);     % Surge deviation level 1
sw_e = max_se + offset_sw(1);     % Sway deviation level 1

su3_e = offset_su(3)-max_se;     % Surge deviation level 3
sw3_e = offset_sw(3)-max_se;     % Sway deviation level 3

su2_e = offset_su(2);            % Surge deviation level 2
sw2_e = offset_sw(2);            % Sway deviation level 2

% Init values
k = 1;
l = 0;
x_e(k) = -ew_e;
y_e(k) = 0;
y_em(k) = 0;
x1_pe(k) = -sw_e;
y1_pe(k) = 0;
y1_pem(k) = 0;
x2_pe(k) = -sw2_e;
y2_pe(k) = 0;
y2_pem(k) = 0;
x3_pe(k) = -sw3_e;
y3_pe(k) = 0;
y3_pem(k) = 0;
h = 0.01;

%% Sensor error ellipse
while x_e < ew_e
    k = k+1;
    x_e(k) = x_e(k-1) + h;
    y_e(k) = sqrt(abs(ns_e^2 - (ns_e^2/ew_e^2)*x_e(k)^2));
    y_em(k) = -y_e(k); %sqrt(abs(ns_e^2 - (ns_e^2/ew_e^2)*x_e(k)^2));
end

%% Rotated sensor error ellipse to body
while l<k
    l=l+1;
    x_be(l) = cos(psi)*x_e(l) + sin(psi)*y_e(l);
    x_bem(l) = cos(psi)*x_e(l) + sin(psi)*y_em(l);
    y_be(l) = -sin(psi)*x_e(l) + cos(psi)*y_e(l);
    y_bem(l) = -sin(psi)*x_e(l) + cos(psi)*y_em(l);
end

%% Position deviation ellipse level 1
k = 1;
while x1_pe < sw_e
    k = k+1;
    x1_pe(k) = x1_pe(k-1) + h;
    y1_pe(k) = sqrt(abs(su_e^2 - (su_e^2/sw_e^2)*x1_pe(k)^2));
    y1_pem(k) = -y1_pe(k);
end

%% Position deviation ellipse body fixed
l = 0;
while l<k
    l=l+1;
    x1_pd(l) = cos(psi)*x1_pe(l) + sin(psi)*y1_pe(l);
    x1_pdm(l) = cos(psi)*x1_pe(l) + sin(psi)*y1_pem(l);
    y1_pd(l) = -sin(psi)*x1_pe(l) + cos(psi)*y1_pe(l);
    y1_pdm(l) = -sin(psi)*x1_pe(l) + cos(psi)*y1_pem(l);
%     r1_pe(l) = sqrt(x1_pd(l)^2  + y1_pd(l)^2);
%     r1_pem(l) = sqrt(x1_pdm(l)^2  + y1_pdm(l)^2);    
end

%% Position deviation ellipse level 2
k = 1;
while x2_pe < sw2_e
    k = k+1;
    x2_pe(k) = x2_pe(k-1) + h;
    y2_pe(k) = sqrt(abs(su2_e^2 - (su2_e^2/sw2_e^2)*x2_pe(k)^2));
    y2_pem(k) = -y2_pe(k);
end

%% Position deviation ellipse 2 body fixed
l = 0;
while l<k
    l=l+1;
    x2_pd(l) = cos(psi)*x2_pe(l) + sin(psi)*y2_pe(l);
    x2_pdm(l) = cos(psi)*x2_pe(l) + sin(psi)*y2_pem(l);
    y2_pd(l) = -sin(psi)*x2_pe(l) + cos(psi)*y2_pe(l);
    y2_pdm(l) = -sin(psi)*x2_pe(l) + cos(psi)*y2_pem(l);
%     r2_pe(l) = sqrt(x2_pd(l)^2  + y2_pd(l)^2);
%     r2_pem(l) = sqrt(x2_pdm(l)^2  + y2_pdm(l)^2);    
end
%% Position deviation ellipse level 3
k = 1;
while x3_pe < sw3_e
    k = k+1;
    x3_pe(k) = x3_pe(k-1) + h;
    y3_pe(k) = sqrt(abs(su3_e^2 - (su3_e^2/sw3_e^2)*x3_pe(k)^2));
    y3_pem(k) = -y3_pe(k);
end

%% Position deviation ellipse 3body fixed
l = 0;
while l<k
    l=l+1;
    x3_pd(l) = cos(psi)*x3_pe(l) + sin(psi)*y3_pe(l);
    x3_pdm(l) = cos(psi)*x3_pe(l) + sin(psi)*y3_pem(l);
    y3_pd(l) = -sin(psi)*x3_pe(l) + cos(psi)*y3_pe(l);
    y3_pdm(l) = -sin(psi)*x3_pe(l) + cos(psi)*y3_pem(l);
%     r3_pe(l) = sqrt(x3_pd(l)^2  + y3_pd(l)^2);
%     r3_pem(l) = sqrt(x3_pdm(l)^2  + y3_pdm(l)^2);    
end

%% Main axes
x_min = floor(min(min(x3_pdm),min(x3_pe)));
y_min = floor(min(min(y3_pdm),min(y3_pd)));
min_axe = min(x_min,y_min);
x_min = min_axe;
y_min = min_axe;
x_max = -x_min;
y_max = -y_min;
ew_axe = x_min:0.1:x_max;
ns_axe = y_min:0.1:y_max;

for n = 1:length(ew_axe)
    y_su(n) = tan(pi/2-psi)*ew_axe(n);
    y_sw(n) = -tan(psi)*ew_axe(n);
end

%% Plot
% Create figure
figure1 = figure('PaperSize',[20.98 29.68]);
% Create axes
axes('Parent',figure1);
xlim([x_min-1, x_max+1])
ylim([y_min-1, y_max+1])
box('on');
hold on
plot(x_e,y_e,'c',x_e,y_em,'c',x1_pd,y1_pd,'g',x1_pdm,y1_pdm,'g')
plot(x2_pd,y2_pd,'y',x2_pdm,y2_pdm,'y',x3_pd,y3_pd,'r',x3_pdm,y3_pdm,'r')
plot(x_pos,y_pos,'b')
% plot(x_pe,y_pe,'c',x_pe,y_pem,'c',x_pd,y_pd,'g',x_pdm,y_pdm,'g')
plot(ew_axe,0,'k--')
plot(0,ns_axe,'k--')
plot(ew_axe,y_su,'k--')
plot(ew_axe,y_sw,'k--')
% Create xlabel
xlabel('WEST-EAST [m]');
% Create ylabel
ylabel('SOUTH-NORTH [m]');
% Create title
title('Position Performance','FontWeight','demi');
% Create textarrow
annotation(figure1,'textarrow',[0.518 0.518],[0.82 0.92],...
    'TextEdgeColor','none',...
    'String',{'Surge'});
% Create textarrow
annotation(figure1,'textarrow',[0.82 0.908],[0.515 0.515],...
    'TextEdgeColor','none',...
    'String',{'Sway'});
% Create textbox
annotation(figure1,'textbox','String',{Str},'FontWeight','bold',...
    'FontSize',11,...
    'FitBoxToText','off',...
    'Position',[0.18 0.81 0.12 0.09]);
if nargin > 7
    annotation(figure1,'textbox',...
        'String',{L1Str,L2Str,L3Str},...
        'FitBoxToText','on',...
        'Position',[0.71 0.55 0.175 0.15]);
    annotation(figure1,'textbox',...
        'String',{JpStr,JtsuStr,JtswStr},...
        'FitBoxToText','on',...
        'Position',[0.7 0.72 0.2 0.18]);
end
if nargin > 13
    annotation(figure1,'textbox',...
        'String',{seaState,weight,w_surge,w_sway},...
        'FitBoxToText','on',...
        'Position',[0.14 0.6 0.19 0.19]);
end
hold off
